<template>
  <div>
    <Echart
      :id="id"
      ref="barChart"
      :options="options"
      :height="height"
      :width="width"
      @click="mapClick"
    ></Echart>
  </div>
</template>

<script>
import Echart from "@/common/echart";

export default {
  name: "mapechart",
  props: {
    id: {
      type: String,
      default: "barChartH",
    },
    width: {
      type: String,
      default: "100%",
    },
    height: {
      type: String,
      default: "100px",
    },
    cdata: {
      type: Array,
      default: () => [],
    },
    organData: {
      type: Array,
      default: () => [],
    },
  },
  data() {
    return {
      options: {},
    };
  },
  components: {
    Echart,
  },
  watch: {
    cdata: {
      handler(newData) {
        this.resetOption(newData, this.organData);
      },
      immediate: true,
      deep: true,
    },
    organData: {
      handler(newData) {
        this.resetOption(this.cdata, newData);
      },
      immediate: true,
      deep: true,
    },
  },
  methods: {
    mapClick(data) {
      this.$emit("click", data);
    },

    resetOption(newData, organData) {
      let xData = [];
      let dataArr=[];

      //比较捐献案例的年份长度和器官案例的年份长度，取最大值
      if(newData.length > organData.length){
        xData = newData.map((item) => item.year);
      }
      else{
        xData = organData.map((item) => item.year);
      }
      for(let k = 0; k < xData.length; k++){
        let singleData = {
          year:xData[k],
          donoatenum:0,
          organnum:0,
        }
        let organIndex = organData.findIndex(item => item.year == xData[k]);
        if(organIndex > -1){
          singleData.organnum = organData[organIndex].num;
        }
        let donoateIndex = newData.findIndex(item => item.year == xData[k]);
        if(donoateIndex > -1){
          singleData.donoatenum = newData[donoateIndex].num
        }
        //填充默认数据
        if(singleData.year =="2019"){
          singleData.donoatenum = 239;
          singleData.organnum = 720;
        }
        else if(singleData.year =="2020"){
          singleData.donoatenum = 281;
          singleData.organnum = 884;
        }
        else if(singleData.year =="2021"){
          singleData.donoatenum = 318;
          singleData.organnum = 1103;
        }
        dataArr.push(singleData);
      }
      this.options = {
        tooltip: {
          trigger: "axis",
          axisPointer: {
            type: "shadow",
          },
        },
        legend: {
          show: true,
          top:"-2"
        },
        grid: {
          left: "6%",
          top: "26%",
          right: "4%",
          bottom: "0%",
          containLabel: true,
        },
        xAxis: {
          type: "category",
          data: xData,
          axisLabel: {
            interval: 0,
            color: "#fff",
            // rotate:45
            // formatter: function (value) {
            //   return value.split("").join("\n");
            // },
            formatter: function (value) {
              return value ? value + "年" : "";
            },
          },
          axisLine: {
            lineStyle: {
              color: "#9cf1ff",
            },
          },
        },
        yAxis: {
          type: "value",
          axisLine: {
            show: true,
            lineStyle: {
              color: "#07dbff",
              width: 1,
              type: "solid",
            },
          },
          splitLine: {
            show: true,
            lineStyle: {
              color: ['rgba(7,219,255,0.5)'],
              type: "dashed",
            },
          },
          axisLabel: {
            color: "#fff",
          },
        },
        animationDurationUpdate: 1000,
        series: [
          {
            name: "历年捐献案例",
            type: "bar",
            data: dataArr.map((item) => item.donoatenum),
            universalTransition: true,
            barWidth: "30%",
            itemStyle: {
              normal: {
                label: {
                  show: true, //开启显示
                  position: "top", //在上方显示
                  textStyle: {
                    //数值样式
                    color: "#07dbff",
                  },
                },
                color: {
                  type: "linear", // 线性渐变
                  x: 0,
                  y: 0,
                  x2: 0,
                  y2: 1,
                  colorStops: [
                    {
                      offset: 0,
                      color: "#07dbff", // 0%处
                    },
                    {
                      offset: 1,
                      color: "#06455d", // 100%处
                    },
                  ],
                },
              },
            },
          },
          {
            name: "历年捐献器官",
            type: "line",
            data: dataArr.map(item => item.organnum),
            universalTransition: true,
            smooth: false,
            itemStyle: {
              normal: {
                label: {
                  show: true, //开启显示
                  position: "top", //在上方显示
                  textStyle: {
                    //数值样式
                    color: "#fff"
                  }
                }
              }
            },

          }
        ],
      };
    },
    // 开启定时器
    startInterval() {
      const _self = this;
      // 应通过接口获取配置时间，暂时写死5s
      const time = 2000;
      if (this.intervalId !== null) {
        clearInterval(this.intervalId);
      }
      this.intervalId = setInterval(() => {
        _self.reSelectMapRandomArea();
      }, time);
    },
    // 重新随机选中地图区域
    reSelectMapRandomArea() {
      const length = 9;
      this.$nextTick(() => {
        const map = this.$refs.mapChart.chart;
        let index = Math.floor(Math.random() * length);
        while (index === this.preSelectMapIndex || index >= length) {
          index = Math.floor(Math.random() * length);
        }
        map.dispatchAction({
          type: "mapUnSelect",
          seriesIndex: 0,
          dataIndex: this.preSelectMapIndex,
        });
        map.dispatchAction({
          type: "showTip",
          seriesIndex: 0,
          dataIndex: index,
        });
        map.dispatchAction({
          type: "mapSelect",
          seriesIndex: 0,
          dataIndex: index,
        });
        this.preSelectMapIndex = index;
      });
    },
    handleMapRandomSelect() {
      this.$nextTick(() => {
        const map = this.$refs.mapChart.chart;
        const _self = this;
        setTimeout(() => {
          _self.reSelectMapRandomArea();
        }, 0);
        // 移入区域，清除定时器、取消之前选中并选中当前
        map.on("mouseover", function (params) {
          clearInterval(_self.intervalId);
          map.dispatchAction({
            type: "mapUnSelect",
            seriesIndex: 0,
            dataIndex: _self.preSelectMapIndex,
          });
          map.dispatchAction({
            type: "mapSelect",
            seriesIndex: 0,
            dataIndex: params.dataIndex,
          });
          _self.preSelectMapIndex = params.dataIndex;
        });
        // 移出区域重新随机选中地图区域，并开启定时器
        map.on("globalout", function () {
          _self.reSelectMapRandomArea();
          _self.startInterval();
        });
        this.startInterval();
      });
    },
  },
};
</script>
